/////////////////////////////////////////////////////////////////////////////
//
//  GEM Object  (and GEM Real Object)
//
//  A GEMobject is the true C++ object which we should think
//  GEMforms to be made up of.  It has basic abilities similar to
//  the abilities of GEM OBJECTs, but with the ability to be "called back",
//  or "touched", and so to act on this stimulus.  As a base class,
//  a GEMobject does not actually do anything in response - this
//  is for derived classes.
//
//  A GEMrealobject is a C++ version of the standard GEM "OBJECT".  It is
//  constructed to have an identical bit image to that object, and as
//  such is only intended as a bridge between the fixed-sized objects
//  of GEM to the variable-sized objects we like to use in Object
//  Oriented class hierachies.  There is no point deriving from a
//  GEMrealobject, since real GEM forms are still only arrays of OBJECTs.
//
//  This file is Copyright 1992 by Warwick W. Allison,
//  and is freely distributable providing no charge is made.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef GEMO_h
#define GEMO_h

#include "gemf.h"
#include <bool.h>
#include <gemfast.h>


class GEMrealobject : public OBJECT
// Since GEMrealobject is derived from OBJECT, and has no new data fields
// or virtual member functions (necessary?), it should be bit-compatible.
{
public:
	// The following are methods suited to different styles
	// Note that the "void" procedure could return the old value, but
	// this seems dangerous, as they would be too similar to the
	// other inspection-only parameterless methods.
	//  Too many choices?

	bool Selected(bool s)		{ return State(SELECTED,s); }
	bool Selected()				{ return States()&SELECTED; }
	void Select()				{ Selected(TRUE); }
	void Deselect()				{ Selected(FALSE); }
	bool Crossed(bool s)		{ return State(CROSSED,s); }
	bool Crossed()				{ return States()&CROSSED; }
	void Cross()				{ Crossed(TRUE); }
	void Uncross()				{ Crossed(FALSE); }
	bool Checked(bool s)		{ return State(CHECKED,s); }
	bool Checked()				{ return States()&CHECKED; }
	void Check()				{ Checked(TRUE); }
	void Uncheck()				{ Checked(FALSE); }
	bool Disabled(bool s)		{ return State(DISABLED,s); }
	bool Disabled()				{ return States()&DISABLED; }
	void Disable()				{ Disabled(TRUE); }
	void Enable()				{ Disabled(FALSE); }
	bool Outlined(bool s)		{ return State(OUTLINED,s); }
	bool Outlined()				{ return States()&OUTLINED; }
	void Outline()				{ Outlined(TRUE); }
	void Unoutline()			{ Outlined(FALSE); }
	bool Shadowed(bool s)		{ return State(SHADOWED,s); }
	bool Shadowed()				{ return States()&SHADOWED; }
	void Shadow()				{ Shadowed(TRUE); }
	void Unshadow()				{ Shadowed(FALSE); }

	bool Selectable(bool f)		{ return Flag(SELECTABLE,f); }
	bool Selectable()			{ return Flags()&SELECTABLE; }
	bool Default(bool f)		{ return Flag(DEFAULT,f); }
	bool Default()				{ return Flags()&DEFAULT; }
	bool Exit(bool f)			{ return Flag(EXIT,f); }
	bool Exit()					{ return Flags()&EXIT; }
	bool Editable(bool f)		{ return Flag(EDITABLE,f); }
	bool Editable()				{ return Flags()&EDITABLE; }
	bool RadioButton(bool f)	{ return Flag(RBUTTON,f); }
	bool RadioButton()			{ return Flags()&RBUTTON; }
	bool LastObject(bool f)		{ return Flag(LASTOB,f); }
	bool LastObject()			{ return Flags()&LASTOB; }
	bool TouchExit(bool f)		{ return Flag(TOUCHEXIT,f); }
	bool TouchExit()			{ return Flags()&TOUCHEXIT; }
	bool HideTree(bool f)		{ return Flag(HIDETREE,f); }
	bool HideTree()				{ return Flags()&HIDETREE; }
	bool Indirect(bool f)		{ return Flag(INDIRECT,f); }
	bool Indirect()				{ return Flags()&INDIRECT; }

	void MoveTo(short x, short y)		{ ob_x=x; ob_y=y; }

private:
	short State(short s, bool on) { bool t=ob_state&s; if (on)
			ob_state|=s; else ob_state&=~s; return t; }
	short States(short s)		{ return ob_state&s; }
	short States()				{ return ob_state; }

	short Flag(short f, bool on) { bool t=ob_flags&f; if (on)
			ob_flags|=f; else ob_flags&=~f; return t;}
	short Flags(short f)		{ return ob_flags&f; }
	short Flags()				{ return ob_flags; }
};

class GEMobject
// A GEMobject represents a true extension of the OBJECT concept.
// But, because GEM uses ARRAYS of OBJECTs, we cannot extend them :-(.
// We make do by coping functionality.
{
public:
	GEMobject(int RSCindex, class GEMform*);

	bool Selected(bool s)		{ return me->Selected(s); }
	bool Selected()				{ return me->Selected(); }
	void Select()				{ me->Select(); }
	void Deselect()				{ me->Deselect(); }
	bool Crossed(bool s)		{ return me->Crossed(s); }
	bool Crossed()				{ return me->Crossed(); }
	void Cross()				{ me->Cross(); }
	void Uncross()				{ me->Uncross(); }
	bool Checked(bool s)		{ return me->Checked(s); }
	bool Checked()				{ return me->Checked(); }
	void Check()				{ me->Check(); }
	void Uncheck()				{ me->Uncheck(); }
	bool Disabled(bool s)		{ return me->Disabled(s); }
	bool Disabled()				{ return me->Disabled(); }
	void Disable()				{ me->Disable(); }
	void Enable()				{ me->Enable(); }
	bool Outlined(bool s)		{ return me->Outlined(s); }
	bool Outlined()				{ return me->Outlined(); }
	void Outline()				{ me->Outline(); }
	void Unoutline()			{ me->Unoutline(); }
	bool Shadowed(bool s)		{ return me->Shadowed(s); }
	bool Shadowed()				{ return me->Shadowed(); }
	void Shadow()				{ me->Shadow(); }
	void Unshadow()				{ me->Unshadow(); }

	bool Selectable(bool f)		{ return me->Selectable(f); }
	bool Selectable()			{ return me->Selectable(); }
	bool Default(bool f)		{ return me->Default(f); }
	bool Default()				{ return me->Default(); }
	bool Exit(bool f)			{ return me->Exit(f); }
	bool Exit()					{ return me->Exit(); }
	bool Editable(bool f)		{ return me->Editable(f); }
	bool Editable()				{ return me->Editable(); }
	bool RadioButton(bool f)	{ return me->RadioButton(f); }
	bool RadioButton()			{ return me->RadioButton(); }

	bool LastObject(bool f)		{ return me->LastObject(f); }
	bool LastObject()			{ return me->LastObject(); }

	bool TouchExit(bool f)		{ return me->TouchExit(f); }
	bool TouchExit()			{ return me->TouchExit(); }
	bool HideTree(bool f)		{ return me->HideTree(f); }
	bool HideTree()				{ return me->HideTree(); }
	bool Indirect(bool f)		{ return me->Indirect(f); }
	bool Indirect()				{ return me->Indirect(); }

	bool ContainsPoint(int x, int y);
	void MoveTo(short x, short y)		{ me->MoveTo(x,y); }

	void Redraw();

	// Call backs...
	virtual void touch(int x, int y);

protected:
	GEMrealobject* me;
	class GEMform* form;
	friend class GEMform;
	int myindex;
};

#endif
